#ifndef CPPE_FILE_HPP
#define CPPE_FILE_HPP

#include <string>
#include <fstream>

#include <sys/stat.h>	//stat

#include <cppe/cppe_exception.hpp>
#include <cppe/flags.hpp>
#include <cppe/permissions.hpp>

namespace cppe { 

/**@brief
 * file is a OO wrapper to the file functions on unix based systems.
 */

class file  {
	public:          
	//Exception classes
	class open_exception : public cppe_exception {
		public:
		open_exception(const std::string& m) : cppe_exception(m) {}
	};
	class permission_exception : public cppe_exception {
		public:
		permission_exception(const std::string& m) : cppe_exception(m) {}
	};
	class read_exception : public cppe_exception {
		public:
		read_exception(const std::string& m) : cppe_exception(m) {}
	};
	class write_exception : public cppe_exception {
		public:
		write_exception(const std::string& m) : cppe_exception(m) {}
	};
	class not_found_exception : public cppe_exception {
		public:
		not_found_exception(const std::string& m) : cppe_exception(m) {}
	};
	
	/**@brief 
	 * Constructor.
	 * @param filename The file this class handles.
	 * @param create_if_not_found if true the file will be created if it is not found.
	 * @throws open_exception if there was a problem opening/creating the file.
	 * @throws permission_exception if we are not allowed to open the file.
	 * @throws not_found_exception if the file doesn't exist and create_if_not_found == false.
	 */
	file(const std::string& filename, bool create_if_not_found = false);
	
	/**@brief 
	 * Read the entire file.
	 * @param read_to The container to read to.
	 * @throws open_exception if there was a problem opening the file.
	 * @throws read_exception if there was a problem reading the file.
	 */
	template<typename T> void read(T &read_to) const;
	
	/**@brief 
	 * Constructor.
	 * @param filename The file this class handles.
	 * @param create_if_not_found if true the file will be created if it is not found.
	 * @throws open_exception if there was a problem opening the file.
	 * @throws write_exception if there was a problem writing the file.
	 */
	template<typename T> void write(const T &write_from, bool append = false);
	
	void user(const std::string& user);
	std::string user() const;
	
	void group(const std::string& group);
	std::string group() const;
	
	void permissions(const cppe::permissions &perms);
	cppe::permissions permissions() const;
	
	void flags(const cppe::flags &flgs);
	cppe::flags flags() const;
	
	unsigned long size() const { return _fileinfo.st_size; }
	
	private:
	void update_info();
	
	struct stat _fileinfo;
	std::string _filename;
};

template<typename T> void file::read(T &read_to) const {
	using namespace std;
	ifstream o;
	o.open(_filename.c_str());
	if(!o) 
		throw open_exception(_filename);

	string input;
	while(o.good()) {
		getline(o, input);
		read_to.push_back(input);
	}
	if(!o.eof()) {
		o.close();	
		throw read_exception(_filename);
	}
	
	read_to.pop_back();	//Last string is always an empty string
	o.close();	
}

template<typename T> void file::write(const T &write_from, bool append) {
	using namespace std;
	ofstream f;
	if(append)
		f.open(_filename.c_str(), ios::app);
	else
		f.open(_filename.c_str(), ios::trunc);
	if(!f) 
		throw open_exception(_filename);

	typename T::const_iterator index = write_from.begin();
	for(;index != write_from.end(); ++index){
		f<<*index<<"\n";
	}
	if(!f) {
		f.close();	
		throw write_exception(_filename);
	}
	f.close();
}				

inline void file::update_info() {
	if(stat(_filename.c_str(), &_fileinfo) == -1) 
		throw open_exception(_filename);
}

inline cppe::permissions file::permissions() const {
	return cppe::permissions(_fileinfo.st_mode);
}

inline void file::permissions(const cppe::permissions &perms) {
	if(chmod(_filename.c_str(), perms) < 0) 
		throw permission_exception("Error setting permissions on: " + _filename);
	
	update_info();
}

inline cppe::flags file::flags() const {
	return cppe::flags(_fileinfo.st_flags);
}

inline void file::flags(const cppe::flags &flgs) {
	if(chflags(_filename.c_str(), flgs) < 0) {
		throw permission_exception("Error setting flags on: " + _filename);
	}
	
	update_info();
}

} //End namespace cppe

#endif
