#ifndef CPPE_PARA_READER_HPP
#define CPPE_PARA_READER_HPP

#include <string>
#include <map>
#include <set>
#include <vector>
#include <limits>

#include <cppe/cppe_exception.hpp>

namespace cppe {

/**@brief
 * This class provide easy handling of the parameters (flag groups and values)
 * given to the program on the commandline.
 * Flag groups are words were the first char is a - (or whatever flag_starter is set to). 
 * E.x.: -a or -aux
 * A flag can have 0 or more values connected to it, however only one flag in
 * a group may take values and that flag has to be at the end of the group.
 * Values that no flag claims are said to be flagless. By default all values are
 * flagless.
 * Let's see an exsample were 'a' take 0, 'f' takes 2 and 'g' takes 1.
 * ./program value1 -af value2 value3 value4 -gvalue5 value6
 * In the above sample value1, value4 and value6 are flagless.
 * If you support flags like "d" and "d2", then be sure to remove "d" first.
 */

class para_reader {
	public:
	
	/**@brief
	 * Exception class which is throw if values are wanted for a flag,
	 * that is not the last in it's group.
	 */
	class not_last_exception : public cppe_exception {
		public:
		not_last_exception(const std::string &message) : cppe_exception(message) {}
	};
	
	/**@brief
	 * Exception class which is throw if a word only has '-' (or whatever flag_starter is set to).
	 */
	class syntax_error_exception : public cppe_exception {
		public:
		syntax_error_exception(const std::string &message) : cppe_exception(message) {}
	};
	
	/**@brief 
	 * The constructor.
	 * @pre No prefix condition.
	 * @post para_reader has it's own list of the parameters, so feel free to edit argv.
	 * @param argc Number of parameters.
	 * @param argv The parameters.
	 * @throws syntax_error_exception if there a word with only "-" in it.
	 */
	para_reader(int argc, char *argv[], char flag_starter = '-');
	
	/**@brief 
	 * Search the list of flags to determin if \c flag is present.
	 * @pre No prefix condition.
	 * @post The flag found is removed from the list.
	 * @param flag The flag that should be searched for.
	 * @return True if the flag is found, otherwise false.
	 */
	bool find_flag(const std::string &flag);

	/**@brief 
	 * Search the list of flags to determin if \c flag is present.
	 * @pre No prefix condition.
	 * @post The flag and value found is removed from the list.
	 * @param flag The flag that should be searched for.
	 * @param value Will contain the value or "" if there wasn't one.
	 * @return True if the flag is found, otherwise false.
	 * @throws no_last_exception if \c flag is not the last in the group.
	 */
	bool find_flag(const std::string &flag, std::string &value);
	
	/**@brief 
	 * Search the list of flags to determin if \c flag is present.
	 * @pre No prefix condition.
	 * @post The flag and values found is removed from the list.
	 * @param flag The flag that should be searched for.
	 * @param values Will contain the values (if there are any).
	 * @param nov The Number Of Values which should be returned.
	 * @return True if the flag is found, otherwise false.
	 * @throws no_last_exception if \c flag is not the last in the group and nov > 0.
	 */
	template<typename T> bool find_flag(const std::string &flag, T &values, int nov = std::numeric_limits<int>::max());

	/**@brief 
	 * Get the next flagless value.
	 * @pre It's recommended to remove all flags first!
	 * @post The value found is removed from the list.
	 * @param Value The next flagless value found or an empty string if no value is found.
	 */
	void flagless_value(std::string &value);
	
	/**@brief 
	 * Get a list of flags still in the list.
	 * @param flags Will contain the flags still not used
	 */
	template<typename T> void flags_left(T &flags) const;

	/**@brief 
	 * Get a list of flags still in the list.
	 * @param values Will contain the values still not used
	 */
	template<typename T> void values_left(T &values) const;
	
	private:
	typedef std::map<int, std::string> flags_type;
	typedef std::map<int, std::string> values_type;
	typedef std::set<int> last_flag_removed_type;
	/**Holds all the flags */
	flags_type _flags;
	/**Holds all the values*/
	values_type _values;
	/**Hold a list of all flag groups, where the last flag in line has been removed */
	last_flag_removed_type _last_flag_removed;
};

inline para_reader::para_reader(int argc, char *argv[],  char flag_starter) {
	int i = 1;	//i = 1 because number 0 is the programs name
	for(;i < argc; ++i) {
		if(argv[i][0] == flag_starter) {	//Is it a flag group?
			if(argv[i][1] == 0)	//There's only a flag starter
				throw syntax_error_exception("Something must come after " + flag_starter);
			std::string value(argv[i]);
			_flags[i] = value.substr(1);	//Remove the flag starter and save the flag group
		}
		else
			_values[i] = argv[i];	//It's a value
	}
}

inline bool para_reader::find_flag(const std::string &flag) {
	std::vector<std::string> tmp;	//Never used, but is needed to call the function below.
	return find_flag(flag, tmp, 0);
}

inline bool para_reader::find_flag(const std::string &flag, std::string &value) {
	std::vector<std::string> tmp;
	if(!find_flag(flag, tmp, 1))	//Find the flag
		return false;		//Flag was not found
	if(tmp.size() == 1)		//Is there a value for the flag?
		value = tmp.front();
	return true;
}

template<typename T> bool para_reader::find_flag(const std::string &flag, T &values, int nov) {
	flags_type::iterator i = _flags.begin();
	for(;i != _flags.end(); ++i) {		//Search throuh all flag groups
		size_t pos = i->second.find(flag);
		if(pos == std::string::npos)		//The flag was not in that group
			continue;
			
		if(nov > 0) {			//Are values wanted?
			//See if the first flag in the group has already been removed
			if(_last_flag_removed.find(i->first) != _last_flag_removed.end())
				throw not_last_exception(i->second + " was not the last flag in the group");
				
			//Read 'nov' or as many as possible values
			for(int j = 1; nov > 0; --nov, ++j) {
				//Get the next word in line
				values_type::iterator value = _values.find(i->first + j);
				if(value == _values.end())		//There isn't any more values for that flag
					break;
				values.push_back(value->second);	//Add the value
				_values.erase(value);			//And remove it from the list of flagless values
			}
		}
		
		if(pos + flag.size() == i->second.size())	//Check if the flag is at the end of the flag_group
			_last_flag_removed.insert(i->first);

		i->second.erase(pos, flag.size());	//Remove the flag
		if(i->second.size() == 0)		//Was it the last flag? Remove from _last_flag_removed also?
			_flags.erase(i);
		
		return true;
	}
	return false;
}

inline void para_reader::flagless_value(std::string &value) {
	values_type::iterator i = _values.begin();
	if(i != _values.end()) {	//Is there any values left?
		value = i->second;
		_values.erase(i);
	}
}
	
template<typename T> void para_reader::flags_left(T &flags) const {
	flags_type::const_iterator i = _flags.begin();
	for(;i != _flags.end(); ++i)
		flags.push_back(i->second);
}

template<typename T> void para_reader::values_left(T &values) const {
	values_type::const_iterator i = _values.begin();
	for(;i != _values.end(); ++i)
		values.push_back(i->second);
}

} //End namespace fsm

#endif
