#include <sys/types.h>
#include <unistd.h>
#include <pwd.h>
#include <grp.h>
#include <errno.h>

#include <cppe/file.hpp>
#include <cppe/string_helpers.hpp>

namespace cppe {

using namespace std;

file::file(const std::string& filename, bool create_if_not_found) : _filename(filename) {
	if(stat(_filename.c_str(), &_fileinfo) == 0) 
		return;
	
	if(errno == EACCES) {
		throw permission_exception(_filename);
	}
	else if(errno == EIO) {
		throw open_exception(_filename);
	}
	else if(errno == ENOENT) {
		if(!create_if_not_found)	//Should I create it?
			throw not_found_exception(_filename);
		fstream f(_filename.c_str(), ios::out | ios::trunc);
		if(!f)
			throw open_exception(_filename);
		f.close();
		update_info();
	}
	else {
		throw not_found_exception(_filename);
	}
}

std::string file::user() const {
	struct passwd *info = getpwuid(_fileinfo.st_uid);
	if(info == 0) {
		string exception = _filename + ": unknown uid: " + int_to_string(_fileinfo.st_uid);
		throw not_found_exception(exception);
	}
	
	return info->pw_name;
}

void file::user(const std::string& user) {
	struct passwd *info = getpwnam(user.c_str());
	if(info == 0) {
		string exception = _filename + ": unknown user: " + user;
		throw not_found_exception(exception);
	}
	
	if(chown(_filename.c_str(), info->pw_uid, _fileinfo.st_gid) < 0) {
		string exception = _filename + ": Error setting user: " + user;
		throw permission_exception(exception);
	}
	
	update_info();
}

std::string file::group() const {
	struct group *gr = getgrgid(_fileinfo.st_gid);
	if(gr == 0) {
		string exception = _filename + ": unknown gid: " + int_to_string(_fileinfo.st_gid);
		throw not_found_exception(exception);
	}
	
	return gr->gr_name;
}

void file::group(const std::string& groupname) {
	struct group *gr = getgrnam(groupname.c_str());
	if(gr == 0) {
		string exception = _filename + ": unknown group: " + groupname;
		throw not_found_exception(exception);
	}

	if(chown(_filename.c_str(), _fileinfo.st_uid, gr->gr_gid) < 0) {
		string exception = _filename + ": Error setting group: " + groupname;
		throw permission_exception(exception);
	}
	
	update_info();
}

} //End namespace cppe
