#ifndef EDITFILE_CONF_READER_HPP
#define EDITFILE_CONF_READER_HPP

#include <cppe/comment_filter.hpp>
#include <cppe/field.hpp>
#include <cppe/para_reader.hpp>
#include <cppe/output.hpp>
#include <cppe/syslog.hpp>

#include <editfile/editfile_exception.hpp>

namespace editfile {

class conf_reader {
	public:
	class conf_reader_exception : public editfile_exception {
		public:
		conf_reader_exception(const std::string &message) : editfile_exception(message) {}
	};

	bool add() const { return _add; }
	bool remove() const { return _remove; }
	bool comment_out() const { return _comment_out; }
	bool autodetect_separator() const { return _autodetect;}
	char separator() const { return _separator; }
	const std::string& file() const { return _file; }
	const std::string& keyword() const { return _keyword; }
	const std::string& value() const { return _value; }
	const std::string& comment() const { return _comment; }
	cppe::comment_filter& comment_filter() { return _filter; }
	void process(int argc, char *argv[]);

	static conf_reader& inst() {
		static conf_reader myself;
		return myself;
	}

	private:
	conf_reader() : _add(true), _remove(false), _comment_out(false), _autodetect(true), _separator('\t') {}

	void process_o_flag(cppe::para_reader &pr);	//Overwrite only.
	void process_d_flag(cppe::para_reader &pr);	//Disable autodetection of the '=' separator.
	void process_s_flag(cppe::para_reader &pr);	//Specify separator.
	void process_c_flag(cppe::para_reader &pr);	//Comma separated list of comment starters.
	void process_r_flag(cppe::para_reader &pr);	//Remove the keyword from the file.
	void process_q_flag(cppe::para_reader &pr);	//Make the program only output error messages and log to syslog.
	void process_file(cppe::para_reader &pr);	//Find the file to use.
	void process_option(cppe::para_reader &pr);	//Find the keyword (and value) to use.

	std::string _file;
	std::string _keyword;
	std::string _value;
	std::string _comment;
	cppe::comment_filter _filter;
	bool _add;
	bool _remove;
	bool _comment_out;
	bool _autodetect;
	char _separator;
};

inline void conf_reader::process_o_flag(cppe::para_reader &pr) {
	if(pr.find_flag("o"))
		_add = false;
}

inline void conf_reader::process_d_flag(cppe::para_reader &pr) {
	if(pr.find_flag("d"))
		_autodetect = false;
}

inline void conf_reader::process_s_flag(cppe::para_reader &pr) {
	std::string separator;
	if(pr.find_flag("s", separator)) {
		if(separator.size() != 1)
			throw conf_reader_exception("Invalid separatorer - a separator is one char.");
		_separator = separator[0];
		_autodetect = false;
	}
}

inline void conf_reader::process_r_flag(cppe::para_reader &pr) {
	if(pr.find_flag("r")) {
		_remove = true;
		_add = false;
	}
}

inline void conf_reader::process_q_flag(cppe::para_reader &pr) {
	if(pr.find_flag("q")) {
		cppe::output::inst().base(new cppe::syslog("editfile"));
		cppe::output::inst().verbose(false);
	}
}

inline void conf_reader::process_file(cppe::para_reader &pr) {
	pr.flagless_value(_file);
	if(_file.empty())
		throw conf_reader_exception("No file specified");
}

} //End namespace editfile

#endif
